!zone irqs
  
.irq2_line                         = $5a
.irq3_line                         = $74
.irq4_line                         = $8e
.irq5_line                         = $9f
.irq1_line                         = $ff

irq_init:
  sei
  lda #$7f
  sta $dc0d  ;disable timer interrupts which can be generated by the two CIA chips
  sta $dd0d  ;the kernal uses such an interrupt to flash the cursor and scan the keyboard, so we better
             ;stop it.

  lda $dc0d  ;by reading this two registers we negate any pending CIA irqs.
  lda $dd0d  ;if we don't do this, a pending CIA irq might occur after we finish setting up our irq.
             ;we don't want that to happen.

  lda #$01   ;this is how to tell the VICII to generate a raster interrupt
  sta $d01a

  lda #.irq1_line   ;this is how to tell at which rasterline we want the irq to be triggered
  sta $d012

  lda #%00011011‬  ;as there are more than 256 rasterlines, the topmost bit of $d011 serves as
  sta $d011  ;the 8th bit for the rasterline we want our irq to be triggered.
             ;here we simply set up a character screen, leaving the topmost bit 0.

  lda #$35   ;we turn off the BASIC and KERNAL rom here
  sta $01    ;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of
             ;SID/VICII/etc are visible

  lda #<.irq1  ;this is how we set up
  sta $fffe  ;the address of our interrupt code
  lda #>.irq1
  sta $ffff

  cli
  rts

.irq1:
  sta savea
  stx savex
  sty savey
  
  
  lax #$00
  tay
  jsr sidplay
  jsr rotate_hires_press_space
  
  ldx #$02  ;next current_task  
  jsr checkkey
  
  lda #$40+14
  sta SPRITE_DATA_MEM+6
  lda #$2c
  sta $d000+12 ; position x
  lda #$42
  sta $d001+12 ; position y
animate_color1=*+1
  lda #$03
  sta $d027+6 ; color  
  
  lda #$40+15
  sta SPRITE_DATA_MEM+7
  lda #$2c
  sta $d000+14 ; position x
  lda #$42
  sta $d001+14 ; position y
animate_color2=*+1
  lda #$01
  sta $d027+7 ; color  

  lda #$46
  sta SPRITE_DATA_MEM
  lda #$48
  sta $d000 ; position x
  lda #$55
  sta $d001 ; position y
  lda #$06
  sta $d027 ; color  
  
  lda #$46
  sta SPRITE_DATA_MEM+1
  lda #$47
  sta $d000+2 ; position x
  lda #$55
  sta $d001+2 ; position y
animate_color3=*+1  
  lda #$0e
  sta $d027+1 ; color  
  
  lda #$43
  sta SPRITE_DATA_MEM+2
  lda #$46
  sta $d000+4 ; position x
  lda #$61
  sta $d001+4 ; position y
  lda #$0f
  sta $d027+2 ; color  

  lda #$42
  sta SPRITE_DATA_MEM+3
  lda #$48
  sta $d000+6 ; position x
  lda #$50
  sta $d001+6 ; position y
  lda #$1c
  sta $d027+3 ; color  

  lda #$41
  sta SPRITE_DATA_MEM+4
  lda #$48
  sta $d000+8 ; position x
  lda #$5f
  sta $d001+8 ; position y
  lda #$0a
  sta $d027+4 ; color

  lda #$44
  sta SPRITE_DATA_MEM+5
  lda #$48
  sta $d000+10 ; position x
  lda #$74
  sta $d001+10 ; position y
  lda #$0a
  sta $d027+5 ; color
  
  jsr set_chars_speedcode  
  
  lda #.irq2_line
  sta $d012

  lda #<.irq2
  sta $fffe
  lda #>.irq2
  sta $ffff  

  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti
  
.irq2:
  sta savea
  stx savex
  sty savey
  
  lda #$45
  sta SPRITE_DATA_MEM+6
  lda #$48
  sta $d000+12 ; position x
  lda #$65
  sta $d001+12 ; position y
  lda #$0c
  sta $d027+6 ; color  
  
  lda #$4d
  sta SPRITE_DATA_MEM+7
  lda #$58
  sta $d000+14 ; position x
  lda #$77
  sta $d001+14 ; position y
  lda #$0a
  sta $d027+7 ; color  
  
  jsr credits_cycle
  
  lda #.irq3_line
  sta $d012

  lda #<.irq3
  sta $fffe
  lda #>.irq3
  sta $ffff  

  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti
  
.irq3:
  sta savea
  stx savex
  sty savey
  
  lda #$47
  sta SPRITE_DATA_MEM
  lda #$30
  sta $d000 ; position x
  lda #$7a
  sta $d001 ; position y
  lda #$0c
  sta $d027 ; color  
  
  lda #$4a
  sta SPRITE_DATA_MEM+1
  lda #$58
  sta $d000+2 ; position x
  lda #$7a
  sta $d001+2 ; position y
  lda #$0c
  sta $d027+1 ; color  
  
  lda #$4c
  sta SPRITE_DATA_MEM+2
  lda #$48
  sta $d000+4 ; position x
  lda #$89
  sta $d001+4 ; position y
  lda #$0a
  sta $d027+2 ; color 

  lda #$49
  sta SPRITE_DATA_MEM+3
  lda #$34
  sta $d000+6 ; position x
  lda #$81
  sta $d001+6 ; position y
  lda #$0a
  sta $d027+3 ; color    
  
  lda #.irq4_line
  sta $d012
  
  lda #<.irq4
  sta $fffe
  lda #>.irq4
  sta $ffff  

  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti  

.irq4:
  sta savea
  stx savex
  sty savey

  lda #$40+8
  sta SPRITE_DATA_MEM+4
  lda #$30
  sta $d000+8 ; position x
  lda #$8f
  sta $d001+8 ; position y
  lda #$0c
  sta $d027+4 ; color

  lda #$40+11
  sta SPRITE_DATA_MEM+5
  lda #$5d
  sta $d000+10 ; position x
  lda #$8f
  sta $d001+10 ; position y
  lda #$0c
  sta $d027+5 ; color
  
  lda #$40+16
  sta SPRITE_DATA_MEM+6
  lda #$78
  sta $d000+12 ; position x
  lda #$90
  sta $d001+12 ; position y
  lda #$05
  sta $d027+6 ; color  
  
  lda #$40+17
  sta SPRITE_DATA_MEM+7
  lda #$48
  sta $d000+14 ; position x
  lda #$95
  sta $d001+14 ; position y
  lda #$06
  sta $d027+7 ; color  
  
  lda #.irq5_line
  sta $d012

  lda #<.irq5
  sta $fffe
  lda #>.irq5
  sta $ffff  
  
  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti  
  
.irq5:
  sta savea
  stx savex
  sty savey

  lda #$40+18
  sta SPRITE_DATA_MEM+3
  lda #$58
  sta $d000+6 ; position x
  lda #$a2
  sta $d001+6 ; position y  
  lda #$0a
  sta $d027+3 ; color  
  
  lda #$40+19
  sta SPRITE_DATA_MEM+1
  lda #$58
  sta $d000+2 ; position x
  lda #$a2
  sta $d001+2 ; position y
  lda #$0c
  sta $d027+1 ; color  

  lda #$00
  cmp current_x
  bne + 
  lda $1683
  cmp .last_pattern
  beq +
  sta .last_pattern
  ;cmp #$09
  ;bne +
  jsr force_animation
+  
  jsr animate_hires
  
  lda #.irq1_line
  sta $d012

  lda #<.irq1
  sta $fffe
  lda #>.irq1
  sta $ffff  

  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti 
  
.last_pattern:
  !byte $00  