!zone irqs2
  
.irq1_line                         = $00
.irq2_line                         = $79
.irq3_line                         = $80

irq_init2:
  sei
  lda #$7f
  sta $dc0d  ;disable timer interrupts which can be generated by the two CIA chips
  sta $dd0d  ;the kernal uses such an interrupt to flash the cursor and scan the keyboard, so we better
             ;stop it.

  lda $dc0d  ;by reading this two registers we negate any pending CIA irqs.
  lda $dd0d  ;if we don't do this, a pending CIA irq might occur after we finish setting up our irq.
             ;we don't want that to happen.

  lda #$01   ;this is how to tell the VICII to generate a raster interrupt
  sta $d01a

  lda #.irq1_line   ;this is how to tell at which rasterline we want the irq to be triggered
  sta $d012

  lda #%00011011‬  ;as there are more than 256 rasterlines, the topmost bit of $d011 serves as
  sta $d011  ;the 8th bit for the rasterline we want our irq to be triggered.
             ;here we simply set up a character screen, leaving the topmost bit 0.

  lda #$35   ;we turn off the BASIC and KERNAL rom here
  sta $01    ;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of
             ;SID/VICII/etc are visible

  lda #<.irq1  ;this is how we set up
  sta $fffe  ;the address of our interrupt code
  lda #>.irq1
  sta $ffff

  cli
  rts

.irq1:
  sta savea
  stx savex
  sty savey


  lda #$0e
  sta $d020
  
  lda #%10000000       ; charmem is at $0000 screenmem is at $2000
  sta $d018
  
  lda #$18             ; activate Multicolor-Textmode 
  sta $d016
  
  lax #$00
  tay
  jsr sidplay
  
  lda #$0e
  sta $d020
  lda #$0f 
  sta $d021
  lda #$0e 
  sta $d022
  lda #$0c 
  sta $d023

  lda #%11011000
  sta VIC_COLOR_CONFIG

  ldx #$01  ;next current_task
  jsr checkkey

  jsr scroll
  
  jsr shift_colors
  
  lda #.irq2_line
  sta $d012

  lda #<.irq2
  sta $fffe
  lda #>.irq2
  sta $ffff  

  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti 
  
.irq2:
  sta savea
  stx savex
  sty savey
  
  jsr waitstable  
  nop
  nop
  nop
  nop
  nop
  nop  
  lda #$06
  sta $d020
  lda #$0e
  sta $d021
  lda #$06 
  sta $d022
  lda #$04 
  sta $d023

  lda #%11011000
  sta VIC_COLOR_CONFIG
  
  lda #.irq3_line
  sta $d012

  lda #<.irq3
  sta $fffe
  lda #>.irq3
  sta $ffff  

  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti 
    
.irq3:
  sta savea
  stx savex
  sty savey
  
  jsr waitstable  
  jsr animate_water

  jsr waitstable  
  lda #$06
  sta $d021
  
  lda #%10000010       ; charmem is at $0800 screenmem is at $2000
  sta $d018

;  lda #$08             ; activate Hires-Textmode 
;  sta $d016  
  lda d016_cache
  sta $d016  
  
  asl $d019
  lda #.irq1_line
  sta $d012

  lda #<.irq1
  sta $fffe
  lda #>.irq1
  sta $ffff  
  
  lda savea
  ldx savex
  ldy savey
  rti  
  
  